//;============================================================
//;    some initialization and interrupt redirect setup
//;============================================================



initIRQ:
	sei       // ;disable maskable IRQs

	lda #$7f
	sta $dc0d  //;disable timer interrupts which can be generated by the two CIA chips
	sta $dd0d  //;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
           	   //;stop it.

	lda $dc0d  //;by reading this two registers we negate any pending CIA irqs.
	lda $dd0d  //;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
           	   //;we don't want that to happen.

	lda #$01   //;this is how to tell the VICII to generate a raster interrupt
	and #%11101111//hide
	ora #%00000000//hide
	sta $d01a

	lda #mainIrqLine   //;this is how to tell at which rasterline we want the irq to be triggered
	sta $d012

	
	//lda $d011                   // clear high bit of raster line
	//and #$7f
	//sta $d011
	
	//lda #$1b   //;as there are more than 256 rasterlines, the topmost bit of $d011 serves as	
	lda #%00000011
	
	//lda $d011                   
	//and #%11111100
	//ora #%00000011
	sta $d011
	
	//sta $d011  //;the 9th bit for the rasterline we want our irq to be triggered.
           	   //;here we simply set up a character screen, leaving the topmost bit 0.

	lda #$35   //;we turn off the BASIC and KERNAL rom here
	sta $01    //;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
           	   //;SID/VICII/etc are visible

	lda #<mainIrq  //;this is how we set up
	sta $fffe  //;the address of our interrupt code
	lda #>mainIrq
	sta $ffff

	cli        //;enable maskable interrupts again

	//jmp *
	//jmp mainLoop
	
	rts

	
initMenuIRQ:
	sei       // ;disable maskable IRQs

	lda #$7f
	sta $dc0d  //;disable timer interrupts which can be generated by the two CIA chips
	sta $dd0d  //;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
           	   //;stop it.

	lda $dc0d  //;by reading this two registers we negate any pending CIA irqs.
	lda $dd0d  //;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
           	   //;we don't want that to happen.

	lda #$01   //;this is how to tell the VICII to generate a raster interrupt
	sta $d01a

	lda #00   //RASTER LINE AT TOP
	sta $d012

	
	
	//setup vic bank
	
	lda $dd00 ; and #%11111100 ; ora #%00000011	; sta $dd00	
	
	
	//lda $d011                   // clear high bit of raster line
	//and #$7f
	//sta $d011
	
	//lda #$1b   //;as there are more than 256 rasterlines, the topmost bit of $d011 serves as	
	lda #%00000011
	
	//lda $d011                   
	//and #%11111100
	//ora #%00000011
	
	
	
	sta $d011
	
	//sta $d011  //;the 9th bit for the rasterline we want our irq to be triggered.
           	   //;here we simply set up a character screen, leaving the topmost bit 0.

	lda #$35   //;we turn off the BASIC and KERNAL rom here
	sta $01    //;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
           	   //;SID/VICII/etc are visible

	lda #<menuIrq0  //;this is how we set up
	sta $fffe  //;the address of our interrupt code
	lda #>menuIrq0
	sta $ffff

	cli        //;enable maskable interrupts again

	//jmp *
	//jmp mainLoop
	
	rts	
	
	
initIntroIRQ:
	sei       // ;disable maskable IRQs

	lda #$7f
	sta $dc0d  //;disable timer interrupts which can be generated by the two CIA chips
	sta $dd0d  //;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
           	   //;stop it.

	lda $dc0d  //;by reading this two registers we negate any pending CIA irqs.
	lda $dd0d  //;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
           	   //;we don't want that to happen.

	lda #$01   //;this is how to tell the VICII to generate a raster interrupt
	sta $d01a

	lda #000   //RASTER LINE AT TOP
	sta $d012

	//for hires		
	//lda #$3b sta $d011	
	//lda #$08 sta $d016
	
	
	
	//setup vic bank
	
	//16384
	lda $dd00 ; and #%11111100 ; ora #%00000010	; sta $dd00	
	//at $c000
	
	//for multicolor
	lda #$3b ; sta $d011
	lda #$18 ; sta $d016
	
	//lda $dd00 and #%11111100 ora #%00000001	sta $dd00	
	//2	xxxxxx01	3276849151	$8000$BFFF	CHARS Yes, at 3686440959 $9000$9FFF
	
	//set bit3 of $d018 to point +8192
	//colors at 49152
	lda #%01111000	; sta $d018
	//lda #$78 ; sta $d018
	
	//bitmap at +8192
	
	//sta $d011  //;the 9th bit for the rasterline we want our irq to be triggered.
           	   //;here we simply set up a character screen, leaving the topmost bit 0.

	lda #$35   //;we turn off the BASIC and KERNAL rom here
	sta $01    //;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
           	   //;SID/VICII/etc are visible

	lda #<introIrq  //;this is how we set up
	sta $fffe  //;the address of our interrupt code
	lda #>introIrq
	sta $ffff

	cli        //;enable maskable interrupts again

	//jmp *
	//jmp mainLoop
	
	rts	
	
	
initIntroTextIRQ:
	sei       // ;disable maskable IRQs

	lda #$7f
	sta $dc0d  //;disable timer interrupts which can be generated by the two CIA chips
	sta $dd0d  //;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
           	   //;stop it.

	lda $dc0d  //;by reading this two registers we negate any pending CIA irqs.
	lda $dd0d  //;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
           	   //;we don't want that to happen.

	lda #$01   //;this is how to tell the VICII to generate a raster interrupt
	sta $d01a

	lda #000   //RASTER LINE AT TOP
	sta $d012

		

	lda #$35   //;we turn off the BASIC and KERNAL rom here
	sta $01    //;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
           	   //;SID/VICII/etc are visible

	lda #<introTextIrq  //;this is how we set up
	sta $fffe  //;the address of our interrupt code
	lda #>introTextIrq
	sta $ffff

	cli        //;enable maskable interrupts again

	//jmp *
	//jmp mainLoop
	
	rts		
	
/*
           sei         //; set interrupt disable flag
                    
           ldy #$7f    //; $7f = %01111111
           sty $dc0d   //; Turn off CIAs Timer interrupts
           sty $dd0d   //; Turn off CIAs Timer interrupts
           lda $dc0d   //; cancel all CIA-IRQs in queue/unprocessed
           lda $dd0d   //; cancel all CIA-IRQs in queue/unprocessed
                     

           lda #<irq   //; point IRQ Vector to our custom irq routine
           ldx #>irq 
           sta $314    //; store in $314/$315
           stx $315   

           lda #00    //; trigger first interrupt at row zero
           sta $d012

           lda $d011   //; Bit#0 of $d011 is basically...
           and #$7f    //; ...the 9th Bit for $d012
           sta $d011   //; we need to make sure it is set to zero 

           
           lda #$01    //; Set Interrupt Request Mask...
           sta $d01a   //; ...we want IRQ by Rasterbeam
           
           cli         //; clear interrupt disable flag
           
           rts
*/         


//;============================================================
//;    custom interrupt routine
//;============================================================
/*
irq:       


 	  sta atemp+1
      stx xtemp+1
      sty ytemp+1

      
      
      //	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
      //     	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
      //     	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
      //     	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
            
	//jsr music.play	
	//jsr $1006	
      
      
      lsr $d019    //;as stated earlier this might fail only on exotic HW like c65 etc.
                   //;lda #$ff sta $d019 is equally fast, but uses two more bytes and
                   //;trashes A

atemp: lda #$00
xtemp: ldx #$00
ytemp: ldy #$00


	  rti
*/
		
//;Being all kernal irq handlers switched off we have to do more work by ourselves.
//;When an interrupt happens the CPU will stop what its doing, store the status and return address
//;into the stack, and then jump to the interrupt routine. It will not store other registers, and if
//;we destroy the value of A/X/Y in the interrupt routine, then when returning from the interrupt to
//;what the CPU was doing will lead to unpredictable results (most probably a crash). So we better
//;store those registers, and restore their original value before reentering the code the CPU was
//;interrupted running.

//;If you won't change the value of a register you are safe to not to store / restore its value.
//;However, it's easy to screw up code like that with later modifying it to use another register too
//;and forgetting about storing its state.

//;The method shown here to store the registers is the most orthodox and most failsafe.

/*
	pha        //;store register A in stack
	txa
	pha        //;store register X in stack
	tya
	pha        //;store register Y in stack
	
			dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
		
	lda #$ff   //;this is the orthodox and safe way of clearing the interrupt condition of the VICII.
	sta $d019  //;if you don't do this the interrupt condition will be present all the time and you end
           //;up having the CPU running the interrupt code all the time, as when it exists the
           //;interrupt, the interrupt request from the VICII will be there again regardless of the
           //;rasterline counter.

           //;it's pretty safe to use inc $d019 (or any other rmw instruction) for brevity, they
           //;will only fail on hardware like c65 or supercpu. c64dtv is ok with this though.

	pla
	tay       // ;restore register Y from stack (remember stack is FIFO: First In First Out)
	pla
	tax       // ;restore register X from stack
	pla       // ;restore register A from stack

	rti      //  ;Return From Interrupt, this will load into the Program Counter register the address
          // ;where the CPU was when the interrupt condition arised which will make the CPU continue
          // ;the code it was interrupted at also restores the status register of the CPU
           
*/

			/*
			 dec $d019        //; acknowledge IRQ

           //jsr colwash      //; jump to color cycling routine
           //jsr play_music   //; jump to play music routine

           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           	dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020 dec $d020
           
//         	:SetBorderColor(2)		
//         	     	:SetBorderColor(3)		
//         	     	     	:SetBorderColor(4)		
//         	     	     	     	:SetBorderColor(5)		
	

           jmp $ea81        //; return to kernel interrupt routine
           
           */